@@ -39,15 +39,20 @@ nanotest(is.list(ssh_config("ssh://remotehost")))
 nanotesterr(ssh_config("ssh://remotehost", tunnel = TRUE), "must be called in the correct context")
 nanotest(is_mirai_interrupt(r <- mirai:::mk_interrupt_error()))
 nanotestp(r)
+nanotest(is_mirai_error(r <- `class<-`("Error in: testing\n", c("miraiError", "errorValue", "try-error"))))
+nanotestp(r)
+nanotestn(r$stack.trace)
+nanotest(mirai:::.DollarNames.miraiError(NULL, "s") == "stack.trace")
+nanotest(mirai:::is.promising.mirai())
 nanotestn(nextstream())
 nanotestn(nextget("pid"))
 Sys.sleep(2.5)
 # mirai tests
 if (connection) {
-  n <- 3L
+  n <- function() m
   m <- mirai({
     Sys.sleep(0.1)
-    q <- m + n + 1L
+    q <- m + n() + 2L
     q / m
   }, m = 2L, .args = list(n), .timeout = 2000L)
   nanotest(identical(call_mirai(m), m))
@@ -77,18 +82,18 @@ if (connection) {
   nanotest(is.character(status()[["daemons"]]))
   nanotestz(daemons(0L))
   Sys.sleep(1L)
-  nanotesto(daemons(1L, dispatcher = FALSE, idletime = 500L, timerstart = 1L, cleanup = FALSE, seed = 1546, .compute = "new"))
+  nanotesto(daemons(1L, dispatcher = FALSE, idletime = 500L, timerstart = 1L, cleanup = FALSE, output = TRUE, seed = 1546, .compute = "new"))
   nanotest(is.character(nextget("urls", .compute = "new")))
   nanotest(is.integer(nextstream(.compute = "new")))
   Sys.sleep(1.5)
-  nanotestn(everywhere(list2env(list(b = 2), envir = .GlobalEnv), .compute = "new"))
+  nanotestn(everywhere(list2env(df, envir = .GlobalEnv), .args = list(df), .compute = "new"))
   mn <- mirai("test1", .compute = "new")
   mp <- mirai(b + 1, .compute = "new")
   Sys.sleep(1L)
   nanotest(unresolved(mn$data) || mn$data == "test1")
   nanotest(unresolved(mp$data) || mp$data == 3)
   Sys.sleep(1L)
-  nanotestz(status(.compute = "new")[["connections"]])
+  nanotest(is.integer(status(.compute = "new")[["connections"]]))
   nanotestz(daemons(0L, .compute = "new"))
   Sys.sleep(1L)
 }
@@ -113,6 +118,7 @@ if (connection && .Platform[["OS.type"]] != "windows") {
   Sys.sleep(1L)
 }
 # parallel cluster tests
+nanotestn(tryCatch(register_cluster(), error = function(e) invisible()))
 if (connection) {
   cluster <- make_cluster(1)
   nanotest(inherits(cluster, "miraiCluster"))
@@ -229,7 +235,8 @@ if (connection && .Platform[["OS.type"]] != "windows" && Sys.getenv("NOT_CRAN")
   nanotestn(launch_local(1L))
   Sys.sleep(1L)
   nanotest(grepl("CERTIFICATE", launch_remote(1L), fixed = TRUE))
-  nanotestn(everywhere(list2env(list(b = 2), envir = .GlobalEnv)))
+  q <- quote(list2env(list(b = 2), envir = .GlobalEnv))
+  nanotestn(everywhere(q))
   m <- mirai(b, .timeout = 1000)
   nanotest(call_mirai(m)$data == 2L || is_error_value(m$data))
   nanotestn(saisei(1))
